==> appslandia-common-2.4-sources.jar: com/appslandia/common/jdbc/Sql.java <==
// The MIT License (MIT)
// Copyright © 2015 AppsLandia. All rights reserved.

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package com.appslandia.common.jdbc;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Collections;
import java.util.Map;

import com.appslandia.common.base.CaseInsensitiveMap;
import com.appslandia.common.base.InitializeObject;
import com.appslandia.common.utils.AssertUtils;

/**
 *
 * @author <a href="mailto:haducloc13@gmail.com">Loc Ha</a>
 *
 */
public class Sql extends InitializeObject implements Serializable {
	private static final long serialVersionUID = 1L;

	private String sql;
	private Map<String, Integer> arrayLens;

	private String translatedSql;
	private Map<String, int[]> indexesMap;

	public Sql() {
	}

	public Sql(String sql) {
		this.sql = sql;
	}

	public Sql arrayParam(String parameterName, int length) {
		assertNotInitialized();
		AssertUtils.assertTrue(length > 0, "length is required.");

		if (this.arrayLens == null) {
			this.arrayLens = new CaseInsensitiveMap<>();
		}
		this.arrayLens.put(parameterName, length);
		return this;
	}

	public Sql sql(String sql) {
		assertNotInitialized();
		this.sql = sql;
		return this;
	}

	@Override
	protected void init() throws Exception {
		AssertUtils.assertNotNull(this.sql, "sql is required.");
		translateSql();
	}

	private void translateSql() {
		StringBuilder sb = new StringBuilder(this.sql);
		Map<String, int[]> indexesMap = new CaseInsensitiveMap<>();

		int i = 0;
		int index = 0;
		while (true) {
			int j = i;
			while (j < sb.length() - 1 && sb.charAt(j) != '@') {
				j++;
			}

			// @parameter
			if (j < sb.length() - 1 && Character.isLetter(sb.charAt(j + 1))) {
				int k = j + 1;
				while (k < sb.length() && (Character.isLetterOrDigit(sb.charAt(k)) || sb.charAt(k) == '_')) {
					k++;
				}
				String paramName = sb.substring(j + 1, k);
				sb.delete(j, k);

				Integer arrayLen = (this.arrayLens == null) ? null : this.arrayLens.get(paramName);
				int len = (arrayLen == null) ? 1 : arrayLen;

				// ?
				for (int subIdx = 0; subIdx < len; subIdx++) {
					if (subIdx == 0) {
						sb.insert(j, '?');
					} else {
						sb.insert(j, "?, ");
					}
				}

				// indexes
				for (int subIdx = 0; subIdx < len; subIdx++) {
					String name = (arrayLen == null) ? paramName : paramName + subIdx;
					int[] indexes = indexesMap.get(name);

					if (indexes == null) {
						indexes = new int[] { ++index };
					} else {
						indexes = Arrays.copyOf(indexes, indexes.length + 1);
						indexes[indexes.length - 1] = ++index;
					}
					indexesMap.put(name, indexes);
				}

				i = j + len;
			} else if (j < sb.length() - 1) {
				throw new IllegalArgumentException("sql is invalid.");
			} else {
				break;
			}
		}

		this.translatedSql = sb.toString();
		this.indexesMap = !indexesMap.isEmpty() ? indexesMap : Collections.emptyMap();
	}

	public String getSql() {
		initialize();
		return this.sql;
	}

	public String getTranslatedSql() {
		initialize();
		return this.translatedSql;
	}

	public int[] getIndexes(String parameterName) {
		initialize();
		int[] indexes = this.indexesMap.get(parameterName);
		if (indexes == null) {
			throw new IllegalArgumentException("Parameter is required (name=" + parameterName + ")");
		}
		return indexes;
	}

	public int getArrayLen(String parameterName) {
		initialize();
		Integer len = (this.arrayLens != null) ? this.arrayLens.get(parameterName) : null;
		if (len == null) {
			throw new IllegalArgumentException("Array parameter is not registered (name=" + parameterName + ")");
		}
		return len;
	}
}

==> appslandia-common-3.2-sources.jar: com/appslandia/common/jdbc/Sql.java <==
// The MIT License (MIT)
// Copyright © 2015 AppsLandia. All rights reserved.

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package com.appslandia.common.jdbc;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import com.appslandia.common.base.InitializeObject;
import com.appslandia.common.utils.AssertUtils;

/**
 *
 * @author <a href="mailto:haducloc13@gmail.com">Loc Ha</a>
 *
 */
public class Sql extends InitializeObject implements Serializable {
	private static final long serialVersionUID = 1L;

	private String sql;
	private Map<String, Integer> arrayLens;

	private String translatedSql;
	private Map<String, int[]> indexesMap;

	public Sql() {
	}

	public Sql(String sql) {
		this.sql = sql;
	}

	public Sql arrayParam(String parameterName, int length) {
		assertNotInitialized();
		AssertUtils.assertTrue(length > 0, "length is required.");

		if (this.arrayLens == null) {
			this.arrayLens = new HashMap<>();
		}
		this.arrayLens.put(parameterName, length);
		return this;
	}

	public Sql sql(String sql) {
		assertNotInitialized();
		this.sql = sql;
		return this;
	}

	@Override
	protected void init() throws Exception {
		AssertUtils.assertNotNull(this.sql, "sql is required.");
		translateSql();
	}

	private void translateSql() {
		StringBuilder sb = new StringBuilder(this.sql);
		Map<String, int[]> indexesMap = new HashMap<>();

		int i = 0;
		int index = 0;
		while (true) {
			int j = i;
			while (j < sb.length() - 1 && sb.charAt(j) != '@') {
				j++;
			}

			// @parameter
			if (j < sb.length() - 1 && Character.isLetter(sb.charAt(j + 1))) {
				int k = j + 1;
				while (k < sb.length() && (Character.isLetterOrDigit(sb.charAt(k)) || sb.charAt(k) == '_')) {
					k++;
				}
				String paramName = sb.substring(j + 1, k);
				sb.delete(j, k);

				Integer arrayLen = (this.arrayLens == null) ? null : this.arrayLens.get(paramName);
				int len = (arrayLen == null) ? 1 : arrayLen;

				// ?
				for (int subIdx = 0; subIdx < len; subIdx++) {
					if (subIdx == 0) {
						sb.insert(j, '?');
					} else {
						sb.insert(j, "?, ");
					}
				}

				// indexes
				for (int subIdx = 0; subIdx < len; subIdx++) {
					String name = (arrayLen == null) ? paramName : paramName + subIdx;
					int[] indexes = indexesMap.get(name);

					if (indexes == null) {
						indexes = new int[] { ++index };
					} else {
						indexes = Arrays.copyOf(indexes, indexes.length + 1);
						indexes[indexes.length - 1] = ++index;
					}
					indexesMap.put(name, indexes);
				}

				i = j + len;
			} else if (j < sb.length() - 1) {
				throw new IllegalArgumentException("sql is invalid.");
			} else {
				break;
			}
		}

		this.translatedSql = sb.toString();
		this.indexesMap = !indexesMap.isEmpty() ? indexesMap : Collections.emptyMap();
	}

	public String getSql() {
		initialize();
		return this.sql;
	}

	public String getTranslatedSql() {
		initialize();
		return this.translatedSql;
	}

	public int[] getIndexes(String parameterName) {
		initialize();
		int[] indexes = this.indexesMap.get(parameterName);
		if (indexes == null) {
			throw new IllegalArgumentException("Parameter is required (name=" + parameterName + ")");
		}
		return indexes;
	}

	public int getArrayLen(String parameterName) {
		initialize();
		Integer len = (this.arrayLens != null) ? this.arrayLens.get(parameterName) : null;
		if (len == null) {
			throw new IllegalArgumentException("Array parameter is not registered (name=" + parameterName + ")");
		}
		return len;
	}
}
